@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2022 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2024 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it
  *   and/or modify it under the terms of the GNU General Public License as
@@ -97,6 +97,17 @@ our_safe_strdup(const char *str, const char *funcname, int line, const char *fil
     return newstr;
 }
 
+pcap_t*
+tcpr_pcap_open(const char *path, char *ebuf)
+{
+#ifdef HAVE_PCAP_OPEN_OFFLINE_WITH_TSTAMP_PRECISION
+    return pcap_open_offline_with_tstamp_precision(path, PCAP_TSTAMP_PRECISION_NANO, ebuf);
+#else
+    return pcap_open_offline(path, ebuf);
+#endif
+}
+
+
 /**
  * calls free and sets to NULL.
  */
@@ -211,7 +222,7 @@ our_safe_pcap_next_ex(pcap_t *pcap,
 void
 packet_stats(const tcpreplay_stats_t *stats)
 {
-    struct timeval diff;
+    struct timespec diff;
     COUNTER diff_us;
     COUNTER bytes_sec = 0;
     u_int32_t bytes_sec_10ths = 0;
@@ -221,8 +232,8 @@ packet_stats(const tcpreplay_stats_t *stats)
     COUNTER pkts_sec = 0;
     u_int32_t pkts_sec_100ths = 0;
 
-    timersub(&stats->end_time, &stats->start_time, &diff);
-    diff_us = TIMEVAL_TO_MICROSEC(&diff);
+    timessub(&stats->end_time, &stats->start_time, &diff);
+    diff_us = TIMESPEC_TO_MICROSEC(&diff);
 
     if (diff_us && stats->pkts_sent && stats->bytes_sent) {
         COUNTER bytes_sec_X10;
@@ -251,18 +262,19 @@ packet_stats(const tcpreplay_stats_t *stats)
         pkts_sec_100ths = pkts_sec_X100 % 100;
     }
 
-    if (diff_us >= 1000 * 1000)
+    if (diff_us >= 1000 * 1000) {
         printf("Actual: " COUNTER_SPEC " packets (" COUNTER_SPEC " bytes) sent in %zd.%02zd seconds\n",
                stats->pkts_sent,
                stats->bytes_sent,
                (ssize_t)diff.tv_sec,
-               (ssize_t)(diff.tv_usec / (10 * 1000)));
-    else
+               (ssize_t)(diff.tv_nsec / (10 * 1000000)));
+    } else {
         printf("Actual: " COUNTER_SPEC " packets (" COUNTER_SPEC " bytes) sent in %zd.%06zd seconds\n",
                stats->pkts_sent,
                stats->bytes_sent,
                (ssize_t)diff.tv_sec,
-               (ssize_t)diff.tv_usec);
+               (ssize_t)diff.tv_nsec / 1000);
+    }
 
     if (mb_sec >= 1)
         printf("Rated: %llu.%1u Bps, %llu.%02u Mbps, %llu.%02u pps\n",
@@ -295,7 +307,7 @@ packet_stats(const tcpreplay_stats_t *stats)
  * @return: string containing date, or -1 on error
  */
 int
-format_date_time(struct timeval *when, char *buf, size_t len)
+format_date_time(struct timespec *when, char *buf, size_t len)
 {
     struct tm *tm;
     char tmp[64];
@@ -306,8 +318,8 @@ format_date_time(struct timeval *when, char *buf, size_t len)
     if (!tm)
         return -1;
 
-    strftime(tmp, sizeof tmp, "%Y-%m-%d %H:%M:%S.%%06u", tm);
-    return snprintf(buf, len, tmp, when->tv_usec);
+    strftime(tmp, sizeof tmp, "%Y-%m-%d %H:%M:%S.%%09u", tm);
+    return snprintf(buf, len, tmp, when->tv_nsec);
 }
 
 /**
