@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2022 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2024 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it
  *   and/or modify it under the terms of the GNU General Public License as
@@ -117,7 +117,7 @@ replay_file(tcpreplay_t *ctx, int idx)
 
     /* read from pcap file if we haven't cached things yet */
     if (!ctx->options->preload_pcap) {
-        if ((pcap = pcap_open_offline(path, ebuf)) == NULL) {
+        if ((pcap = tcpr_pcap_open(path, ebuf)) == NULL) {
             tcpreplay_seterr(ctx, "Error opening pcap file: %s", ebuf);
             return -1;
         }
@@ -133,7 +133,7 @@ replay_file(tcpreplay_t *ctx, int idx)
 
     } else {
         if (!ctx->options->file_cache[idx].cached) {
-            if ((pcap = pcap_open_offline(path, ebuf)) == NULL) {
+            if ((pcap = tcpr_pcap_open(path, ebuf)) == NULL) {
                 tcpreplay_seterr(ctx, "Error opening pcap file: %s", ebuf);
                 return -1;
             }
@@ -145,7 +145,7 @@ replay_file(tcpreplay_t *ctx, int idx)
     if (ctx->options->verbose) {
         /* in cache mode, we may not have opened the file */
         if (pcap == NULL)
-            if ((pcap = pcap_open_offline(path, ebuf)) == NULL) {
+            if ((pcap = tcpr_pcap_open(path, ebuf)) == NULL) {
                 tcpreplay_seterr(ctx, "Error opening pcap file: %s", ebuf);
                 return -1;
             }
@@ -214,26 +214,26 @@ replay_two_files(tcpreplay_t *ctx, int idx1, int idx2)
 
     /* read from first pcap file if we haven't cached things yet */
     if (!ctx->options->preload_pcap) {
-        if ((pcap1 = pcap_open_offline(path1, ebuf)) == NULL) {
+        if ((pcap1 = tcpr_pcap_open(path1, ebuf)) == NULL) {
             tcpreplay_seterr(ctx, "Error opening pcap file: %s", ebuf);
             return -1;
         }
         ctx->options->file_cache[idx1].dlt = pcap_datalink(pcap1);
-        if ((pcap2 = pcap_open_offline(path2, ebuf)) == NULL) {
+        if ((pcap2 = tcpr_pcap_open(path2, ebuf)) == NULL) {
             tcpreplay_seterr(ctx, "Error opening pcap file: %s", ebuf);
             return -1;
         }
         ctx->options->file_cache[idx2].dlt = pcap_datalink(pcap2);
     } else {
         if (!ctx->options->file_cache[idx1].cached) {
-            if ((pcap1 = pcap_open_offline(path1, ebuf)) == NULL) {
+            if ((pcap1 = tcpr_pcap_open(path1, ebuf)) == NULL) {
                 tcpreplay_seterr(ctx, "Error opening pcap file: %s", ebuf);
                 return -1;
             }
             ctx->options->file_cache[idx1].dlt = pcap_datalink(pcap1);
         }
         if (!ctx->options->file_cache[idx2].cached) {
-            if ((pcap2 = pcap_open_offline(path2, ebuf)) == NULL) {
+            if ((pcap2 = tcpr_pcap_open(path2, ebuf)) == NULL) {
                 tcpreplay_seterr(ctx, "Error opening pcap file: %s", ebuf);
                 return -1;
             }
@@ -293,7 +293,7 @@ replay_two_files(tcpreplay_t *ctx, int idx1, int idx2)
     if (ctx->options->verbose) {
         /* in cache mode, we may not have opened the file */
         if (pcap1 == NULL) {
-            if ((pcap1 = pcap_open_offline(path1, ebuf)) == NULL) {
+            if ((pcap1 = tcpr_pcap_open(path1, ebuf)) == NULL) {
                 tcpreplay_seterr(ctx, "Error opening pcap file: %s", ebuf);
                 return -1;
             }
