@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2022 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2024 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it
  *   and/or modify it under the terms of the GNU General Public License as
@@ -94,7 +94,7 @@ again:
     ipflags = 0;
     /* not everything has a L3 header, so check for errors.  returns proto in network byte order */
     if ((l2proto = tcpedit_dlt_proto(tcpedit->dlt_ctx, src_dlt, packet, (int)(*pkthdr)->caplen)) < 0) {
-        dbgx(2, "Packet has no L3+ header: %s", tcpedit_geterr(tcpedit));
+        dbgx(2, "Packet " COUNTER_SPEC " has no L3+ header: %s", tcpedit->runtime.packetnum, tcpedit_geterr(tcpedit));
         return TCPEDIT_SOFT_ERROR;
     } else {
         dbgx(2, "Layer 3 protocol type is: 0x%04x", ntohs(l2proto));
@@ -185,8 +185,9 @@ again:
             volatile uint16_t oldval = *((uint16_t *)ip_hdr);
             volatile uint16_t newval;
 
-            ip_hdr->ip_tos = tcpedit->tos;
             newval = *((uint16_t *)ip_hdr);
+            newval = htons((ntohs(newval) & 0xff00) | (tcpedit->tos & 0xff));
+            *((uint16_t *)ip_hdr) = newval;
             csum_replace2(&ip_hdr->ip_sum, oldval, newval);
         }
 
@@ -315,12 +316,20 @@ again:
         }
     }
 
-    /* ensure IP header length is correct */
-    if (ip_hdr != NULL) {
-        fix_ipv4_length(*pkthdr, ip_hdr, l2len);
-        needtorecalc = 1;
-    } else if (ip6_hdr != NULL) {
-        needtorecalc |= fix_ipv6_length(*pkthdr, ip6_hdr, l2len);
+    /* fixhdrlen option ensure IP header length is correct */
+    /* do we need to fix checksums? -- must always do this last! */
+    if (tcpedit->fixhdrlen) {
+        /* ensure IP header length is correct */
+        int changed = 0;
+        if (ip_hdr != NULL) {
+            changed = fix_ipv4_length(*pkthdr, ip_hdr, l2len);
+        } else if (ip6_hdr != NULL) {
+            changed = fix_ipv6_length(*pkthdr, ip6_hdr, l2len);
+        }
+        /* did the packet change? then needtorecalc checksum */
+        if (changed > 0) {
+            needtorecalc |= changed;
+        }
     }
 
     /* do we need to fix checksums? -- must always do this last! */
