@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2022 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2024 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it
  *   and/or modify it under the terms of the GNU General Public License as
@@ -147,6 +147,9 @@ typedef struct {
 
     uint32_t fuzz_seed;
     uint32_t fuzz_factor;
+
+    /* fix header length */
+    bool fixhdrlen;
 } tcpedit_t;
 
 #ifdef __cplusplus
