@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2022 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2024 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it
  *   and/or modify it under the terms of the GNU General Public License as
@@ -91,7 +91,7 @@ main(int argc, char *argv[])
 
 readpcap:
     /* open the pcap file */
-    if ((options->pcap = pcap_open_offline(OPT_ARG(PCAP), errbuf)) == NULL) {
+    if ((options->pcap = tcpr_pcap_open(OPT_ARG(PCAP), errbuf)) == NULL) {
         close(out_file);
         tcpprep_close(tcpprep);
         errx(-1, "Error opening libpcap: %s", errbuf);
@@ -108,6 +108,7 @@ readpcap:
     switch (pcap_datalink(options->pcap)) {
     case DLT_EN10MB:
     case DLT_LINUX_SLL:
+    case DLT_LINUX_SLL2:
     case DLT_RAW:
     case DLT_C_HDLC:
     case DLT_JUNIPER_ETHER:
@@ -124,6 +125,9 @@ readpcap:
         err(-1, "MAC mode splitting is only supported by DLT_EN10MB packet captures.");
     }
 
+    if (HAVE_OPT(SUPPRESS_WARNINGS))
+        print_warnings = 0;
+
 #ifdef ENABLE_VERBOSE
     if (HAVE_OPT(VERBOSE)) {
         tcpdump_open(&tcpprep->tcpdump, options->pcap);
