@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2022 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2024 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -18,23 +18,23 @@
  *   along with the Tcpreplay Suite.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
+#include "tcpreplay_api.h"
 #include "defines.h"
+#include "config.h"
 #include "common.h"
-
+#include "replay.h"
+#include "send_packets.h"
+#include "sleep.h"
 #include <ctype.h>
+#include <errno.h>
 #include <fcntl.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
+#include <time.h>
 #include <unistd.h>
-#include <errno.h>
-#include <stdarg.h>
-
-#include "tcpreplay_api.h"
-#include "send_packets.h"
-#include "replay.h"
 
 #ifdef TCPREPLAY_EDIT
 #include "tcpreplay_edit_opts.h"
@@ -166,6 +166,7 @@ tcpreplay_post_args(tcpreplay_t *ctx, int argc)
 
     options->loop = OPT_VALUE_LOOP;
     options->loopdelay_ms = OPT_VALUE_LOOPDELAY_MS;
+    options->loopdelay_ns = OPT_VALUE_LOOPDELAY_NS;
 
     if (HAVE_OPT(LIMIT))
         options->limit_send = OPT_VALUE_LIMIT;
@@ -208,6 +209,9 @@ tcpreplay_post_args(tcpreplay_t *ctx, int argc)
         options->maxsleep.tv_nsec = (OPT_VALUE_MAXSLEEP % 1000) * 1000 * 1000;
     }
 
+    if (HAVE_OPT(SUPPRESS_WARNINGS))
+        print_warnings = 0;
+
 #ifdef ENABLE_VERBOSE
     if (HAVE_OPT(VERBOSE))
         options->verbose = 1;
@@ -265,6 +269,15 @@ tcpreplay_post_args(tcpreplay_t *ctx, int argc)
 #endif
     }
 
+    if (HAVE_OPT(XDP)) {
+#ifdef HAVE_LIBXDP
+        options->xdp = 1;
+        ctx->sp_type = SP_TYPE_LIBXDP;
+#else
+         err(-1, "--xdp feature was not compiled in. See INSTALL.");
+#endif
+    }
+
     if (HAVE_OPT(UNIQUE_IP))
         options->unique_ip = 1;
 
@@ -328,74 +341,100 @@ tcpreplay_post_args(tcpreplay_t *ctx, int argc)
         tcpreplay_setwarn(ctx, "%s", "--pktlen may cause problems.  Use with caution.");
     }
 
-    if ((intname = get_interface(ctx->intlist, OPT_ARG(INTF1))) == NULL) {
-        if (!strncmp(OPT_ARG(INTF1), "netmap:", 7) || !strncmp(OPT_ARG(INTF1), "vale", 4))
-            tcpreplay_seterr(ctx, "Unable to connect to netmap interface %s. Ensure netmap module is installed (see INSTALL).",
-                    OPT_ARG(INTF1));
-        else
-            tcpreplay_seterr(ctx, "Invalid interface name/alias: %s", OPT_ARG(INTF1));
+    switch (WHICH_IDX_INTF1) {
+    case INDEX_OPT_WRITE:
+        options->intf1_name = safe_strdup(OPT_ARG(INTF1));
+        ctx->sp_type = SP_TYPE_LIBPCAP_DUMP;
+        /* open interfaces for writing */
+        if ((ctx->intf1 = sendpacket_open(options->intf1_name, ebuf, TCPR_DIR_C2S, ctx->sp_type, ctx)) == NULL) {
+            tcpreplay_seterr(ctx, "Can't open %s: %s", options->intf1_name, ebuf);
+            ret = -1;
+            goto out;
+        }
+        break;
 
-        ret = -1;
-        goto out;
-    }
+    case INDEX_OPT_INTF1:
+        if ((intname = get_interface(ctx->intlist, OPT_ARG(INTF1))) == NULL) {
+            if (!strncmp(OPT_ARG(INTF1), "netmap:", 7) || !strncmp(OPT_ARG(INTF1), "vale", 4)) {
+                tcpreplay_seterr(
+                        ctx,
+                        "Unable to connect to netmap interface %s. Ensure netmap module is installed (see INSTALL).",
+                        OPT_ARG(INTF1));
+            } else {
+                tcpreplay_seterr(ctx, "Invalid interface name/alias: %s", OPT_ARG(INTF1));
+            }
 
-    if (!strncmp(intname, "netmap:", 7) || !strncmp(intname, "vale:", 5)) {
+            ret = -1;
+            goto out;
+        }
+
+        if (!strncmp(intname, "netmap:", 7) || !strncmp(intname, "vale:", 5)) {
 #ifdef HAVE_NETMAP
-        options->netmap = 1;
-        ctx->sp_type = SP_TYPE_NETMAP;
+            options->netmap = 1;
+            ctx->sp_type = SP_TYPE_NETMAP;
 #else
-        tcpreplay_seterr(ctx, "%s", "tcpreplay_api not compiled with netmap support");
-        ret = -1;
-        goto out;
+            tcpreplay_seterr(ctx, "%s", "tcpreplay_api not compiled with netmap support");
+            ret = -1;
+            goto out;
 #endif
-    }
+        }
 
-    options->intf1_name = safe_strdup(intname);
+        options->intf1_name = safe_strdup(intname);
 
-    /* open interfaces for writing */
-    if ((ctx->intf1 = sendpacket_open(options->intf1_name, ebuf, TCPR_DIR_C2S, ctx->sp_type, ctx)) == NULL) {
-        tcpreplay_seterr(ctx, "Can't open %s: %s", options->intf1_name, ebuf);
-        ret = -1;
-        goto out;
-    }
+        /* open interfaces for writing */
+        if ((ctx->intf1 = sendpacket_open(options->intf1_name, ebuf, TCPR_DIR_C2S, ctx->sp_type, ctx)) == NULL) {
+            tcpreplay_seterr(ctx, "Can't open %s: %s", options->intf1_name, ebuf);
+            ret = -1;
+            goto out;
+        }
+#ifdef HAVE_LIBXDP
+        ctx->intf1->batch_size = OPT_VALUE_XDP_BATCH_SIZE;
+#endif
+#if defined HAVE_NETMAP
+        ctx->intf1->netmap_delay = ctx->options->netmap_delay;
+#endif
+
+        ctx->intf1dlt = sendpacket_get_dlt(ctx->intf1);
+
+        if (HAVE_OPT(INTF2)) {
+            if (!HAVE_OPT(CACHEFILE) && !HAVE_OPT(DUALFILE)) {
+                tcpreplay_seterr(ctx, "--intf2=%s requires either --cachefile or --dualfile", OPT_ARG(INTF2));
+                ret = -1;
+                goto out;
+            }
+            if ((intname = get_interface(ctx->intlist, OPT_ARG(INTF2))) == NULL) {
+                tcpreplay_seterr(ctx, "Invalid interface name/alias: %s", OPT_ARG(INTF2));
+                ret = -1;
+                goto out;
+            }
+
+            options->intf2_name = safe_strdup(intname);
+
+            /* open interface for writing */
+            if ((ctx->intf2 = sendpacket_open(options->intf2_name, ebuf, TCPR_DIR_S2C, ctx->sp_type, ctx)) == NULL) {
+                tcpreplay_seterr(ctx, "Can't open %s: %s", options->intf2_name, ebuf);
+            }
 
 #if defined HAVE_NETMAP
-    ctx->intf1->netmap_delay = ctx->options->netmap_delay;
+            ctx->intf2->netmap_delay = ctx->options->netmap_delay;
 #endif
 
-    ctx->intf1dlt = sendpacket_get_dlt(ctx->intf1);
-
-    if (HAVE_OPT(INTF2)) {
-        if (!HAVE_OPT(CACHEFILE) && !HAVE_OPT(DUALFILE)) {
-            tcpreplay_seterr(ctx, "--intf2=%s requires either --cachefile or --dualfile", OPT_ARG(INTF2));
-            ret = -1;
-            goto out;
-        }
-        if ((intname = get_interface(ctx->intlist, OPT_ARG(INTF2))) == NULL) {
-            tcpreplay_seterr(ctx, "Invalid interface name/alias: %s", OPT_ARG(INTF2));
-            ret = -1;
-            goto out;
+            ctx->intf2dlt = sendpacket_get_dlt(ctx->intf2);
+            if (ctx->intf2dlt != ctx->intf1dlt) {
+                tcpreplay_seterr(ctx,
+                                 "DLT type mismatch for %s (%s) and %s (%s)",
+                                 options->intf1_name,
+                                 pcap_datalink_val_to_name(ctx->intf1dlt),
+                                 options->intf2_name,
+                                 pcap_datalink_val_to_name(ctx->intf2dlt));
+                ret = -1;
+                goto out;
+            }
         }
+        break;
 
-        options->intf2_name = safe_strdup(intname);
-
-        /* open interface for writing */
-        if ((ctx->intf2 = sendpacket_open(options->intf2_name, ebuf, TCPR_DIR_S2C, ctx->sp_type, ctx)) == NULL) {
-            tcpreplay_seterr(ctx, "Can't open %s: %s", options->intf2_name, ebuf);
-        }
-
-#if defined HAVE_NETMAP
-        ctx->intf2->netmap_delay = ctx->options->netmap_delay;
-#endif
-
-        ctx->intf2dlt = sendpacket_get_dlt(ctx->intf2);
-        if (ctx->intf2dlt != ctx->intf1dlt) {
-            tcpreplay_seterr(ctx, "DLT type mismatch for %s (%s) and %s (%s)",
-                options->intf1_name, pcap_datalink_val_to_name(ctx->intf1dlt),
-                options->intf2_name, pcap_datalink_val_to_name(ctx->intf2dlt));
-            ret = -1;
-            goto out;
-        }
+    default:
+        assert(false); // shouldn't happen!
     }
 
     if (HAVE_OPT(CACHEFILE)) {
@@ -429,6 +468,15 @@ tcpreplay_close(tcpreplay_t *ctx)
     assert(ctx->options);
     options = ctx->options;
 
+#ifdef HAVE_LIBXDP
+    if (ctx->intf1->handle_type == SP_TYPE_LIBXDP) {
+        free_umem_and_xsk(ctx->intf1);
+        if (ctx->intf2) {
+            free_umem_and_xsk(ctx->intf2);
+        }
+    }
+#endif
+
     safe_free(options->intf1_name);
     safe_free(options->intf2_name);
     sendpacket_close(ctx->intf1);
@@ -463,6 +511,9 @@ tcpreplay_close(tcpreplay_t *ctx)
             intlist = intlistnext;
         }
     }
+
+    /* free --include / --exclude list */
+    free_list(options->list);
 }
 
 /**
@@ -895,24 +946,24 @@ tcpreplay_get_failed(tcpreplay_t *ctx)
 }
 
 /**
- * \brief returns a pointer to the timeval structure of when replay first started
+ * \brief returns a pointer to the timespec structure of when replay first started
  */
-const struct timeval *
+const struct timespec *
 tcpreplay_get_start_time(tcpreplay_t *ctx)
 {
     assert(ctx);
-    TIMEVAL_SET(&ctx->static_stats.start_time, &ctx->stats.start_time);
+    TIMESPEC_SET(&ctx->static_stats.start_time, &ctx->stats.start_time);
     return &ctx->static_stats.start_time;
 }
 
 /**
- * \brief returns a pointer to the timeval structure of when replay finished
+ * \brief returns a pointer to the timespec structure of when replay finished
  */
-const struct timeval *
+const struct timespec *
 tcpreplay_get_end_time(tcpreplay_t *ctx)
 {
     assert(ctx);
-    TIMEVAL_SET(&ctx->static_stats.end_time, &ctx->stats.end_time);
+    TIMESPEC_SET(&ctx->static_stats.end_time, &ctx->stats.end_time);
     return &ctx->static_stats.end_time;
 }
 
@@ -1099,6 +1150,7 @@ out:
     return ret;
 }
 
+static bool apply_loop_delay(tcpreplay_t *ctx);
 /**
  * \brief sends the traffic out the interfaces
  *
@@ -1149,14 +1201,20 @@ tcpreplay_replay(tcpreplay_t *ctx)
             if ((rcode = tcpr_replay_index(ctx)) < 0)
                 return rcode;
             if (ctx->options->loop > 0) {
-                if (!ctx->abort && ctx->options->loopdelay_ms > 0) {
-                    usleep(ctx->options->loopdelay_ms * 1000);
-                    gettimeofday(&ctx->stats.end_time, NULL);
-                }
+                if (apply_loop_delay(ctx))
+                    get_current_time(&ctx->stats.end_time);
 
-                if (ctx->options->stats == 0)
+                if (ctx->options->stats == 0) {
                     packet_stats(&ctx->stats);
+                }
             }
+#ifdef HAVE_LIBXDP
+            sendpacket_t *sp = ctx->intf1;
+            if (sp->handle_type == SP_TYPE_LIBXDP) {
+                sp->xsk_info->tx.cached_prod = 0;
+                sp->xsk_info->tx.cached_cons = sp->tx_size;
+            }
+#endif
         }
     } else {
         while (!ctx->abort) { /* loop forever unless user aborts */
@@ -1171,10 +1229,8 @@ tcpreplay_replay(tcpreplay_t *ctx)
             if ((rcode = tcpr_replay_index(ctx)) < 0)
                 return rcode;
 
-            if (!ctx->abort && ctx->options->loopdelay_ms > 0) {
-                usleep(ctx->options->loopdelay_ms * 1000);
-                gettimeofday(&ctx->stats.end_time, NULL);
-            }
+            if (apply_loop_delay(ctx))
+                get_current_time(&ctx->stats.end_time);
 
             if (ctx->options->stats == 0 && !ctx->abort)
                 packet_stats(&ctx->stats);
@@ -1356,3 +1412,55 @@ int tcpreplay_get_flow_expiry(tcpreplay_t *ctx)
 
     return ctx->options->flow_expiry;
 }
+
+bool
+apply_loop_delay(tcpreplay_t *ctx) {
+    if (!ctx->abort && ctx->options->loopdelay_ms > 0) {
+        usleep(ctx->options->loopdelay_ms * 1000);
+        return true;
+    }
+
+    if (!ctx->abort && ctx->options->loopdelay_ns > 0) {
+        struct timespec nap;
+        nap.tv_sec = 0;
+        nap.tv_nsec = ctx->options->loopdelay_ns;
+        nanosleep_sleep(NULL, &nap, &ctx->stats.end_time, NULL);
+        return true;
+    }
+
+    return false;
+}
+
+#ifdef HAVE_LIBXDP
+void
+delete_xsk_socket(struct xsk_socket *xsk)
+{
+    size_t desc_sz = sizeof(struct xdp_desc);
+    struct xdp_mmap_offsets off;
+    socklen_t optlen;
+    int err;
+
+    if (!xsk) {
+        return;
+    }
+
+    optlen = sizeof(off);
+    err = getsockopt(xsk->fd, SOL_XDP, XDP_MMAP_OFFSETS, &off, &optlen);
+    if (!err) {
+        if (xsk->rx) {
+            munmap(xsk->rx->ring - off.rx.desc, off.rx.desc + xsk->config.rx_size * desc_sz);
+        }
+        if (xsk->tx) {
+            munmap(xsk->tx->ring - off.tx.desc, off.tx.desc + xsk->config.tx_size * desc_sz);
+        }
+    }
+    close(xsk->fd);
+}
+
+void
+free_umem_and_xsk(sendpacket_t *sp)
+{
+    xsk_umem__delete(sp->xsk_info->umem->umem);
+    delete_xsk_socket(sp->xsk_info->xsk);
+}
+#endif /*HAVE_LIBXDP*/
